package de.uni_marburg.powersort.benchmark;

import de.uni_marburg.powersort.data.DataEnum;
import de.uni_marburg.powersort.data.ObjectSupplier;
import de.uni_marburg.powersort.sort.SortEnum;
import org.openjdk.jmh.annotations.Benchmark;
import org.openjdk.jmh.annotations.BenchmarkMode;
import org.openjdk.jmh.annotations.Fork;
import org.openjdk.jmh.annotations.Level;
import org.openjdk.jmh.annotations.Measurement;
import org.openjdk.jmh.annotations.Mode;
import org.openjdk.jmh.annotations.OutputTimeUnit;
import org.openjdk.jmh.annotations.Param;
import org.openjdk.jmh.annotations.Scope;
import org.openjdk.jmh.annotations.Setup;
import org.openjdk.jmh.annotations.State;
import org.openjdk.jmh.annotations.Warmup;

import java.util.concurrent.TimeUnit;

import static de.uni_marburg.powersort.data.ObjectSupplier.EMPTY_ARRAY;

// TODO: The parameters are way too low. Use for debugging only!
/*
 * Benchmark parameters
 */
@Fork(1)
@Warmup(iterations = 1)
@Measurement(iterations = 6)
@BenchmarkMode(Mode.AverageTime)
@OutputTimeUnit(TimeUnit.MILLISECONDS)
/*
 * Benchmark state parameters
 *
 * Quote from JMH:
 * State objects naturally encapsulate the state on which benchmark is working on.
 */
@State(Scope.Benchmark)
public class MainJmh {
    @Param()
    private DataEnum dataEnum;
    @Param()
    private SortEnum sortEnum;

    private ObjectSupplier data = EMPTY_ARRAY;
    /* package-protected */ Object[] workingCopy;

    // TODO: This is inaccurate. How to create and use separate arrays for each warmup x iteration x sortAlgorithm ?
    @Setup(Level.Invocation)
    public void setup() {
        if(Filter.isFiltered(dataEnum, sortEnum)) {
            // This combination of DataEnum and SortEnum should be skipped.
            // We can't tell JMH to not run the benchmark at all,
            // so we'll let it sort an empty list instead ;)
            data = EMPTY_ARRAY;
            return;
        }

        // A new MainJmh object is created for each @Param variation.
        // Then, `data` is `null` again.
        if (data == EMPTY_ARRAY) {
            data = dataEnum.getObjectSupplier();
        }
        // For all warmup and measurement iterations of one @Param variation, the MainJmh object is reused.
        // Thus, we can't just sort `data` directly.
        // Instead, we have to create a copy of it on which the sort algorithm can work.
        // This way, all iterations sort the same input.
        workingCopy = data.getCopy();
    }

    @Benchmark
    public void benchmark() {
        sortEnum.getSortImpl().sort(workingCopy);
    }
}
