package de.uni_marburg.powersort.data;

public enum DataEnum {
    RANDOM_INTEGERS_300M,
    ASCENDING_INTEGERS_300M,
    DESCENDING_INTEGERS_250M,
    ASCENDING_RUNS_1M,
    ASCENDING_RUNS_WITH_OVERLAP_1M;

    public ObjectSupplier getObjectSupplier() {
        // We use a seed to get the same random list every time -> Repeatable benchmarks on same input data!
        // final long seed = 3651660232967549736L; // System.nanoTime() ++ Math.random()
        final long seed = 140506881906827520L; // (long) 'P' * (long) 'O' *(long) 'W' * (long) 'E' * (long) 'R' * (long) 'S' * (long) 'O' * (long) 'R' * (long) 'T';

        int longListSize = 300_000_000; // Any larger: Out of Heap Space. TODO GRADLE config
        int middleListSize = 250_000_000;
        int runs = 1_000;
        int runLength = 1_000;

        return switch (this) {
            case RANDOM_INTEGERS_300M -> new RandomIntegers(longListSize, seed);
            case ASCENDING_INTEGERS_300M -> new AscendingIntegers(longListSize);
            case DESCENDING_INTEGERS_250M -> new DescendingIntegers(middleListSize);

            case ASCENDING_RUNS_1M -> AscendingRuns.newAscendingRuns(runs, runLength, -runLength);
            case ASCENDING_RUNS_WITH_OVERLAP_1M -> AscendingRuns.newAscendingRuns(runs, runLength, (int) (-0.5 * runLength));
        };
    }
}
